/* -*- mode: Java; c-basic-offset: 4 -*- */
/*------------------------------------------------------------------------
 |  FILE            KeyValuePanel.java
 |  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 +----------------------------------------------------------------------*/

package no.shhsoft.soapdebugger;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.LinkedList;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

/**
 *
 *
 * @author Sverre H. Huseby
 *          &lt;<a href="mailto:shh@thathost.com">shh@thathost.com </a>&gt;
 */
final class KeyValuePanel
extends JPanel {
    /*-----------------------------------------------------------------------+
    |  PRIVATE PART                                                          |
    +-----------------------------------------------------------------------*/
    private JPanel panel;

    private ArrayList kvList;

    private LinkedList kvtfList;

    private Component focusComponent = null;

    class KeyValueTextField
    extends JTextField {
        KeyValue kv;

        KeyValueTextField(KeyValue kv) {
            super(kv.value);
            this.kv = kv;
            setCaretPosition(0);
        }
    }

    private void addField(JPanel panel, GridBagLayout gbl,
                          GridBagConstraints gbc, Component field,
                          int gwidth, int gheight,
                          int gx, int wx, int wy) {
        if (gx == 0 && wy == 0) { /* label */
            gbc.anchor = GridBagConstraints.NORTHEAST;
            gbc.fill = GridBagConstraints.NONE;
        } else {
            gbc.anchor = GridBagConstraints.WEST;
            gbc.fill = GridBagConstraints.HORIZONTAL;
        }
        gbc.gridwidth = gwidth;
        gbc.gridheight = gheight;
        gbc.gridx = gx;
        gbc.weightx = wx;
        gbc.weighty = wy;
        gbl.setConstraints(field, gbc);
        panel.add(field);
    }

    /*-----------------------------------------------------------------------+
    |  PUBLIC INTERFACE                                                      |
    +-----------------------------------------------------------------------*/
    public KeyValuePanel() {
        panel = new JPanel();
        setLayout(new GridLayout(1, 1));
        add(new JScrollPane(panel));
    }

    public synchronized void setFrom(ArrayList kvList) {
        KeyValue kv;
        KeyValueTextField kvtf;
        int q;
        GridBagLayout gbl;
        GridBagConstraints gbc;

        if (kvList == null && this.kvList == null)
            return;
        if (kvList != null && this.kvList != null
            && kvList.equals(this.kvList))
            return;
        this.kvList = kvList;

        panel.setVisible(false);
        panel.removeAll();

        gbl = new GridBagLayout();
        gbc = new GridBagConstraints();
        panel.setLayout(gbl);
        gbc.insets = new Insets(2, 5, 2, 5);

        kvtfList = new LinkedList();
        focusComponent = null;
        if (kvList != null)
            for (q = 0; q < kvList.size(); q++) {
                kv = (KeyValue) kvList.get(q);
                kvtf = new KeyValueTextField(kv);
                kvtfList.add(kvtf);
                addField(panel, gbl, gbc, new JLabel(kv.key), 1, 1, 0, 0, 0);
                addField(panel, gbl, gbc, kvtf, GridBagConstraints.REMAINDER,
                        1, GridBagConstraints.RELATIVE, 1, 0);
                if (q == 0 && focusComponent == null)
                    focusComponent = kvtf;
            }
        panel.setVisible(true);
        if (focusComponent != null)
            focusComponent.requestFocus();
    }

    public synchronized void copyBack() {
        int q;
        KeyValueTextField kvtf;

        if (kvtfList == null)
            return;
        for (q = kvtfList.size() - 1; q >= 0; q--) {
            kvtf = (KeyValueTextField) kvtfList.get(q);
            kvtf.kv.value = kvtf.getText();
        }
    }
}
