package no.shhsoft.soapdebugger;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.wsdl.Operation;

import org.apache.axis.wsdl.symbolTable.Parameter;
import org.apache.axis.wsdl.symbolTable.TypeEntry;

/**
 * @author sverrehu
 */
public class MainWindow
extends JFrame
implements ActionListener {
    private SOAPClient sc;
    private javax.swing.JPanel jContentPane = null;

    private JComboBox jComboBox = null;
    private JPanel jPanel = null;
    private JScrollPane jScrollPane = null;
    private JButton goButton = null;
    private JTextArea resultPanel = null;
    private KeyValuePanel keyValuePanel = null;
    private ArrayList keyValues = null;
    private HashMap valueHash = new HashMap();

    public MainWindow(SOAPClient sc) {
        super();
        this.sc = sc;
        initialize();
    }

    private void initialize() {
        setPreferredSize(new Dimension(750, 550));
        setContentPane(getJContentPane());
        setTitle("JFrame");
        getJComboBox().setSelectedIndex(0);
    }

    private javax.swing.JPanel getJContentPane() {
        if (jContentPane == null) {
            jContentPane = new javax.swing.JPanel();
            jContentPane.setLayout(new java.awt.BorderLayout());

            JPanel p = new JPanel();
            p.setLayout(new java.awt.BorderLayout());
            p.add(getJComboBox(), java.awt.BorderLayout.CENTER);
            p.add(getGoButton(), java.awt.BorderLayout.EAST);

            jContentPane.add(p, java.awt.BorderLayout.NORTH);

            JSplitPane sp = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                                           getJPanel(), getResultPanel());
            sp.setDividerLocation(250);
            jContentPane.add(sp, java.awt.BorderLayout.CENTER);
        }
        return jContentPane;
    }

    private JComboBox getJComboBox() {
        if (jComboBox == null) {
            jComboBox = new JComboBox();
            Set operations = sc.getOperations();
            ArrayList al = new ArrayList();
            Iterator i = operations.iterator();
            while (i.hasNext()) {
                Operation o = (Operation) i.next();
                al.add(o.getName());
            }
            Collections.sort(al);
            for (int q = 0; q < al.size(); q++)
                jComboBox.addItem((String) al.get(q));
            jComboBox.addActionListener(this);
        }
        return jComboBox;
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == getJComboBox()) {
            String operationName = (String) getJComboBox().getSelectedItem();
            Vector inNames = new Vector();
            Vector inTypes = new Vector();
            Vector outNames = new Vector();
            getResultPanel().setText("");
            try {
                sc.getParameters(operationName, inNames, inTypes, outNames);
                keyValues = new ArrayList();
                for (int q = 0; q < inNames.size(); q++) {
                    String name = (String) inNames.elementAt(q);
                    TypeEntry type = ((Parameter) inTypes.elementAt(q)).getType();
                    String typeName = type.getQName().getLocalPart();
                    String key = name + " [" + typeName + "]";
                    String value = (String) valueHash.get(key);
                    if (value == null)
                        value = "";
                    keyValues.add(new KeyValue(key, value));
                }
                getKeyValuePanel().setFrom(keyValues);
            } catch (Exception ex) {
                System.err.println("error: " + ex.getMessage());
                ex.printStackTrace();
            }
        } else if (e.getSource() == getGoButton()) {
            getResultPanel().setText("wait...");
            for (int q = 0; q < 100; q++)
                Thread.yield();
            getKeyValuePanel().copyBack();
            String[] args = new String[keyValues.size()];
            for (int q = 0; q < args.length; q++) {
                String key = ((KeyValue) keyValues.get(q)).key;
                String value = ((KeyValue) keyValues.get(q)).value;
                args[q] = value;
                valueHash.put(key, value);
            }
            try {
                HashMap out = sc.invokeMethod((String) getJComboBox().getSelectedItem(), args);
                StringBuffer sb = new StringBuffer();
                sb.append("Call finished.\n");
                Iterator i = out.keySet().iterator();
                while (i.hasNext()) {
                    String key = (String) i.next();
                    String value = (String) out.get(key);
                    sb.append(key);
                    sb.append(": ");
                    sb.append(value);
                    sb.append("\n");
                }
                getResultPanel().setText(sb.toString());
            } catch (Exception ex) {
                getResultPanel().setText("error: " + ex.getMessage());
            }
        }
    }

    private JPanel getJPanel() {
        if (jPanel == null) {
            jPanel = new JPanel();
            jPanel.setLayout(new java.awt.BorderLayout());
            jPanel.add(getJScrollPane(), java.awt.BorderLayout.CENTER);
        }
        return jPanel;
    }

    private JScrollPane getJScrollPane() {
        if (jScrollPane == null) {
            jScrollPane = new JScrollPane(getKeyValuePanel());
        }
        return jScrollPane;
    }

    private KeyValuePanel getKeyValuePanel() {
        if (keyValuePanel == null) {
            keyValuePanel = new KeyValuePanel();
        }
        return keyValuePanel;
    }

    private JTextArea getResultPanel() {
        if (resultPanel == null) {
            resultPanel = new JTextArea();
            resultPanel.setLineWrap(true);
        }
        return resultPanel;
    }

    private JButton getGoButton() {
        if (goButton == null) {
            goButton = new JButton("Go");
            goButton.addActionListener(this);
        }
        return goButton;
    }
}
