package no.shhsoft.soapdebugger;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.wsdl.Binding;
import javax.wsdl.Operation;
import javax.wsdl.Port;
import javax.wsdl.Service;
import javax.wsdl.extensions.soap.SOAPAddress;
import javax.xml.namespace.QName;
import javax.xml.rpc.Call;
import javax.xml.rpc.encoding.Deserializer;
import javax.xml.rpc.encoding.DeserializerFactory;

import org.apache.axis.Constants;
import org.apache.axis.encoding.ser.ElementDeserializer;
import org.apache.axis.encoding.ser.SimpleDeserializer;
import org.apache.axis.utils.XMLUtils;
import org.apache.axis.wsdl.gen.Parser;
import org.apache.axis.wsdl.symbolTable.BaseType;
import org.apache.axis.wsdl.symbolTable.BindingEntry;
import org.apache.axis.wsdl.symbolTable.Parameter;
import org.apache.axis.wsdl.symbolTable.Parameters;
import org.apache.axis.wsdl.symbolTable.ServiceEntry;
import org.apache.axis.wsdl.symbolTable.SymTabEntry;
import org.apache.axis.wsdl.symbolTable.SymbolTable;
import org.apache.axis.wsdl.symbolTable.TypeEntry;
import org.w3c.dom.Element;

public class SOAPClient {

    private Parser wsdlParser = null;

    public SOAPClient(String wsdlURL)
    throws Exception {
        // Start by reading in the WSDL using Parser
        wsdlParser = new Parser();
        System.out.println("Reading WSDL document from '" + wsdlURL + "'");
        wsdlParser.run(wsdlURL);
    }

    public HashMap invokeMethod(String operationName, String[] args)
    throws Exception {
        String operationQName = null;

        Service service = getService();
        Operation operation = null;
        org.apache.axis.client.Service dpf
          = new org.apache.axis.client.Service(wsdlParser, service.getQName());

        Vector inputs = new Vector();
        Port port = selectPort(service.getPorts());
        Binding binding = port.getBinding();
        Call call = dpf.createCall(QName.valueOf(port.getName()), QName
                .valueOf(operationName));
        ((org.apache.axis.client.Call) call).setTimeout(
                                                 new Integer(1200 * 1000));
        ((org.apache.axis.client.Call) call).setProperty(
                ElementDeserializer.DESERIALIZE_CURRENT_ELEMENT, Boolean.TRUE);

        // Output types and names
        Vector outNames = new Vector();

        // Input types and names
        Vector inNames = new Vector();
        Vector inTypes = new Vector();
        getParameters(operationName, inNames, inTypes, outNames);

        if (inNames.size() != args.length)
            throw new RuntimeException("Need " + inNames.size()
                    + " arguments!");

        for (int pos = 0; pos < inNames.size(); ++pos) {
            String arg = args[pos];
            Parameter p = (Parameter) inTypes.get(pos);
            inputs.add(getParamData((org.apache.axis.client.Call) call, p,
                        arg));
        }

        Object ret = call.invoke(inputs.toArray());
        Map outputs = call.getOutputParams();
        HashMap map = new HashMap();

        for (int pos = 0; pos < outNames.size(); ++pos) {
            String name = (String) outNames.get(pos);
            Object value = outputs.get(name);

            if ((value == null) && (pos == 0)) {
                map.put(name, ret);
            } else {
                map.put(name, value);
            }
        }
        for (Iterator it = map.entrySet().iterator(); it.hasNext();) {
            Map.Entry entry = (Map.Entry) it.next();
            String key = (String) entry.getKey();
            Object value = entry.getValue();
            if (value instanceof Element) {
                System.out.println("====== " + key + " ======");
                XMLUtils.ElementToStream((Element) value, System.out);
                System.out.println("=========================");
            } else {
                System.out.println(key + "=" + value);
            }
        }
        return map;
    }

    public void getParameters(String operationName, Vector inNames,
                              Vector inTypes, Vector outNames)
    throws Exception {
        Service service = getService();

        Port port = selectPort(service.getPorts());
        Binding binding = port.getBinding();

        SymbolTable symbolTable = wsdlParser.getSymbolTable();
        BindingEntry bEntry = symbolTable.getBindingEntry(binding.getQName());
        Parameters parameters = null;
        Iterator i = bEntry.getParameters().keySet().iterator();

        Operation operation = null;
        while (i.hasNext()) {
            Operation o = (Operation) i.next();
            if (o.getName().equals(operationName)) {
                operation = o;
                parameters = (Parameters) bEntry.getParameters().get(o);
                break;
            }
        }
        if ((operation == null) || (parameters == null)) {
            throw new RuntimeException(operationName + " was not found.");
        }

        for (int j = 0; j < parameters.list.size(); ++j) {
            Parameter p = (Parameter) parameters.list.get(j);

            if (p.getMode() == 1) { // IN
                inNames.add(p.getQName().getLocalPart());
                inTypes.add(p);
            } else if (p.getMode() == 2) { // OUT
                outNames.add(p.getQName().getLocalPart());
            } else if (p.getMode() == 3) { // INOUT
                inNames.add(p.getQName().getLocalPart());
                inTypes.add(p);
                outNames.add(p.getQName().getLocalPart());
            }
        }

        if (parameters.returnParam != null) {
            // Get the QName for the return Type
            QName returnType = org.apache.axis.wsdl.toJava.Utils
                    .getXSIType(parameters.returnParam);
            QName returnQName = parameters.returnParam.getQName();

            outNames.add(returnQName.getLocalPart());
        }
    }

    private Object getParamData(org.apache.axis.client.Call c, Parameter p,
                                String arg)
    throws Exception {
        // Get the QName representing the parameter type
        QName paramType = org.apache.axis.wsdl.toJava.Utils.getXSIType(p);

        TypeEntry type = p.getType();
        if (type instanceof BaseType && ((BaseType) type).isBaseType()) {
            DeserializerFactory factory = c.getTypeMapping().getDeserializer(
                    paramType);
            Deserializer deserializer = factory
                    .getDeserializerAs(Constants.AXIS_SAX);
            if (deserializer instanceof SimpleDeserializer) {
                return ((SimpleDeserializer) deserializer).makeValue(arg);
            }
        }
        throw new RuntimeException("don't know how to convert '" + arg
                + "' into " + c);
    }

    public Set getOperations() {
        try {
            Service service = getService();
            Port port = selectPort(service.getPorts());
            Binding binding = port.getBinding();

            SymbolTable symbolTable = wsdlParser.getSymbolTable();
            BindingEntry bEntry = symbolTable.getBindingEntry(binding
                    .getQName());
            Parameters parameters = null;
            return bEntry.getOperations();
        } catch (Exception e) {
            System.err.println("error: " + e.getMessage());
            System.exit(1);
        }
        return null; /* make compiler happy */
    }

    public Service getService() {
        HashMap map = wsdlParser.getSymbolTable().getHashMap();
        Iterator iterator = map.entrySet().iterator();

        while (iterator.hasNext()) {
            Map.Entry entry = (Map.Entry) iterator.next();
            QName key = (QName) entry.getKey();
            Vector v = (Vector) entry.getValue();

            for (int i = 0; i < v.size(); ++i) {
                SymTabEntry symTabEntry = (SymTabEntry) v.elementAt(i);

                if (ServiceEntry.class.isInstance(symTabEntry))
                        return ((ServiceEntry) symTabEntry).getService();
                }
        }
        return null;
    }

    public Port selectPort(Map ports)
    throws Exception {
        Iterator valueIterator = ports.keySet().iterator();
        while (valueIterator.hasNext()) {
            String name = (String) valueIterator.next();

            Port port = (Port) ports.get(name);
            List list = port.getExtensibilityElements();

            for (int i = 0; (list != null) && (i < list.size()); i++) {
                Object obj = list.get(i);
                if (obj instanceof SOAPAddress)
                        return port;
            }
        }
        return null;
    }
}
